#![allow(unused, reason = "Temporary.")]
//! goal is a custom ui/menu library that allows simple and declarative menus.
//! bevy ui is annoying.

use std::marker::PhantomData;

use bevy::{platform::collections::HashMap, prelude::*};

mod item;

pub use item::types::*;
pub use item::*;

type ToDo = ();

#[derive(Debug, Clone)]
pub struct Menus<TriggerState: States, NavState: States> {
    start: NavState,
    /// The state needed to trigger this set of menus
    trigger: TriggerState,
    menus: HashMap<NavState, Menu<NavState>>,
}

#[derive(Debug, Clone)]
pub struct Menu<ParentNavState: States> {
    menu_items: Vec<MenuItemInternal<ParentNavState>>,
}

pub mod plugin;

impl<TriggerState: States, NavState: States + Default> Menus<TriggerState, NavState> {
    pub fn new(trigger: TriggerState) -> Self {
        Self {
            trigger,
            start: NavState::default(),
            menus: HashMap::new(),
        }
    }
}

impl<T: States, NavState: States> Menus<T, NavState> {
    pub fn add_menu(mut self, state: NavState, menu: Menu<NavState>) -> Self {
        self.menus.insert(state, menu);
        self
    }
}

impl<ParentNavState: States> Menu<ParentNavState> {
    pub fn new() -> Self {
        Self {
            menu_items: Vec::new(),
        }
    }

    /// Adds `items` in the default `ItemPosition::MainView`.
    pub fn add_items(mut self, items: &[&dyn MenuItem<ParentNavState>]) -> Self {
        self.add_items_positioned(ItemPosition::MainView, items)
    }

    pub fn add_items_positioned(
        mut self,
        position: ItemPosition,
        items: &[&dyn MenuItem<ParentNavState>],
    ) -> Self {
        items
            .iter()
            .map(|it| (*it).item())
            .collect_into(&mut self.menu_items);
        self
    }
}
